@@ -39,14 +39,21 @@
 //!
 //! println!("Input ({}) with 10 decimals: {} vs {})", input, dec, float);
 //! ```
+#![allow(clippy::unreadable_literal)]
+#![allow(clippy::needless_return)]
+#![allow(clippy::suspicious_arithmetic_impl)]
+#![allow(clippy::suspicious_op_assign_impl)]
+#![allow(clippy::redundant_field_names)]
 
-extern crate num_bigint;
+pub extern crate num_bigint;
+pub extern crate num_traits;
 extern crate num_integer;
-extern crate num_traits as traits;
+
 #[cfg(feature = "serde")]
 extern crate serde;
 
 use std::cmp::Ordering;
+use std::convert::TryFrom;
 use std::default::Default;
 use std::error::Error;
 use std::fmt;
@@ -57,8 +64,10 @@ use std::iter::Sum;
 use std::str::{self, FromStr};
 
 use num_bigint::{BigInt, ParseBigIntError, Sign, ToBigInt};
-use num_integer::Integer;
-pub use traits::{FromPrimitive, Num, One, Signed, ToPrimitive, Zero};
+use num_integer::Integer as IntegerTrait;
+pub use num_traits::{FromPrimitive, Num, One, Signed, ToPrimitive, Zero};
+
+const LOG2_10: f64 = 3.321928094887362_f64;
 
 #[macro_use]
 mod macros;
@@ -90,9 +99,9 @@ fn count_decimal_digits(int: &BigInt) -> u64 {
         return 1;
     }
     // guess number of digits based on number of bits in UInt
-    let mut digits = (int.bits() as f64 / 3.3219280949) as u64;
+    let mut digits = (int.bits() as f64 / LOG2_10) as u64;
     let mut num = ten_to_the(digits);
-    while int >= &num {
+    while *int >= num {
         num *= 10u8;
         digits += 1;
     }
@@ -112,16 +121,16 @@ fn get_rounding_term(num: &BigInt) -> u8 {
         return 0;
     }
 
-    let digits = (num.bits() as f64 / 3.3219280949) as u64;
+    let digits = (num.bits() as f64 / LOG2_10) as u64;
     let mut n = ten_to_the(digits);
 
     // loop-method
     loop {
-        if num < &n {
+        if *num < n {
             return 1;
         }
         n *= 5;
-        if num < &n {
+        if *num < n {
             return 0;
         }
         n *= 2;
@@ -155,13 +164,16 @@ impl BigDecimal {
 
     /// Creates and initializes a `BigDecimal`.
     ///
+    /// Decodes using `str::from_utf8` and forwards to `BigDecimal::from_str_radix`.
+    /// Only base-10 is supported.
+    ///
     /// # Examples
     ///
     /// ```
     /// use bigdecimal::{BigDecimal, Zero};
     ///
     /// assert_eq!(BigDecimal::parse_bytes(b"0", 10).unwrap(), BigDecimal::zero());
-    /// // assert_eq!(BigDecimal::parse_bytes(b"f", 16), BigDecimal::parse_bytes(b"16", 10));
+    /// assert_eq!(BigDecimal::parse_bytes(b"13", 10).unwrap(), BigDecimal::from(13));
     /// ```
     #[inline]
     pub fn parse_bytes(buf: &[u8], radix: u32) -> Option<BigDecimal> {
@@ -225,7 +237,7 @@ impl BigDecimal {
 
             // check for "leading zero" in remainder term; otherwise round
             if p < 10 * &r {
-              q += get_rounding_term(&r);
+                q += get_rounding_term(&r);
             }
 
             BigDecimal {
@@ -385,16 +397,14 @@ impl BigDecimal {
 
         // make guess
         let guess = {
-            let log2_10 = 3.32192809488736234787031942948939018_f64;
             let magic_guess_scale = 1.1951678538495576_f64;
-            let initial_guess = (self.int_val.bits() as f64 - self.scale as f64 * log2_10) / 2.0;
+            let initial_guess = (self.int_val.bits() as f64 - self.scale as f64 * LOG2_10) / 2.0;
             let res = magic_guess_scale * initial_guess.exp2();
             if res.is_normal() {
-                BigDecimal::from(res)
+                BigDecimal::try_from(res).unwrap()
             } else {
                 // can't guess with float - just guess magnitude
-                let scale =
-                    (self.int_val.bits() as f64 / -log2_10 + self.scale as f64).round() as i64;
+                let scale = (self.int_val.bits() as f64 / -LOG2_10 + self.scale as f64).round() as i64;
                 BigDecimal::new(BigInt::from(1), scale / 2)
             }
         };
@@ -459,16 +469,14 @@ impl BigDecimal {
 
         // make guess
         let guess = {
-            let log2_10 = 3.32192809488736234787031942948939018_f64;
             let magic_guess_scale = 1.124960491619939_f64;
-            let initial_guess = (self.int_val.bits() as f64 - self.scale as f64 * log2_10) / 3.0;
+            let initial_guess = (self.int_val.bits() as f64 - self.scale as f64 * LOG2_10) / 3.0;
             let res = magic_guess_scale * initial_guess.exp2();
             if res.is_normal() {
-                BigDecimal::from(res)
+                BigDecimal::try_from(res).unwrap()
             } else {
                 // can't guess with float - just guess magnitude
-                let scale =
-                    (self.int_val.bits() as f64 / log2_10 - self.scale as f64).round() as i64;
+                let scale = (self.int_val.bits() as f64 / LOG2_10 - self.scale as f64).round() as i64;
                 BigDecimal::new(BigInt::from(1), -scale / 3)
             }
         };
@@ -487,12 +495,7 @@ impl BigDecimal {
                 max_precision + 1,
             );
             let tmp = tmp + r.double();
-            impl_division(
-                tmp.int_val,
-                &three.int_val,
-                tmp.scale - three.scale,
-                max_precision + 1,
-            )
+            impl_division(tmp.int_val, &three.int_val, tmp.scale - three.scale, max_precision + 1)
         };
 
         // result initial
@@ -533,16 +536,15 @@ impl BigDecimal {
             let bits = self.int_val.bits() as f64;
             let scale = self.scale as f64;
 
-            let log2_10 = 3.32192809488736234787031942948939018_f64;
             let magic_factor = 0.721507597259061_f64;
-            let initial_guess = scale * log2_10 - bits;
+            let initial_guess = scale * LOG2_10 - bits;
             let res = magic_factor * initial_guess.exp2();
 
             if res.is_normal() {
-                BigDecimal::from(res)
+                BigDecimal::try_from(res).unwrap()
             } else {
                 // can't guess with float - just guess magnitude
-                let scale = (bits / log2_10 + scale).round() as i64;
+                let scale = (bits / LOG2_10 + scale).round() as i64;
                 BigDecimal::new(BigInt::from(1), -scale)
             }
         };
@@ -581,6 +583,32 @@ impl BigDecimal {
         return result;
     }
 
+    /// Return number rounded to round_digits precision after the decimal point
+    pub fn round(&self, round_digits: i64) -> BigDecimal {
+        let (bigint, decimal_part_digits) = self.as_bigint_and_exponent();
+        let need_to_round_digits = decimal_part_digits - round_digits;
+        if round_digits >= 0 && need_to_round_digits <= 0 {
+            return self.clone();
+        }
+
+        let mut number = bigint.to_i128().unwrap();
+        if number < 0 {
+            number = -number;
+        }
+        for _ in 0..(need_to_round_digits - 1) {
+            number /= 10;
+        }
+        let digit = number % 10;
+
+        if digit <= 4 {
+            self.with_scale(round_digits)
+        } else if bigint.is_negative() {
+            self.with_scale(round_digits) - BigDecimal::new(BigInt::from(1), round_digits)
+        } else {
+            self.with_scale(round_digits) + BigDecimal::new(BigInt::from(1), round_digits)
+        }
+    }
+
     /// Return true if this number has zero fractional part (is equal
     /// to an integer)
     ///
@@ -589,7 +617,7 @@ impl BigDecimal {
         if self.scale <= 0 {
             true
         } else {
-           (self.int_val.clone() % ten_to_the(self.scale as u64)).is_zero()
+            (self.int_val.clone() % ten_to_the(self.scale as u64)).is_zero()
         }
     }
 
@@ -612,12 +640,7 @@ impl BigDecimal {
             term *= self;
             factorial *= n;
             // ∑ term=x^n/n!
-            result += impl_division(
-                term.int_val.clone(),
-                &factorial,
-                term.scale,
-                117 + precision,
-            );
+            result += impl_division(term.int_val.clone(), &factorial, term.scale, 117 + precision);
 
             let trimmed_result = result.with_prec(105);
             if prev_result == trimmed_result {
@@ -627,6 +650,20 @@ impl BigDecimal {
         }
         return result.with_prec(100);
     }
+
+    #[must_use]
+    pub fn normalized(&self) -> BigDecimal {
+        if self == &BigDecimal::zero() {
+            return BigDecimal::zero();
+        }
+        let (sign, mut digits) = self.int_val.to_radix_be(10);
+        let trailing_count = digits.iter().rev().take_while(|i| **i == 0).count();
+        let trunc_to = digits.len() - trailing_count as usize;
+        digits.truncate(trunc_to);
+        let int_val = BigInt::from_radix_be(sign, &digits, 10).unwrap();
+        let scale = self.scale - trailing_count as i64;
+        BigDecimal::new(int_val, scale)
+    }
 }
 
 #[derive(Debug, PartialEq)]
@@ -685,7 +722,7 @@ impl FromStr for BigDecimal {
     }
 }
 
-#[allow(deprecated)]  // trim_right_match -> trim_end_match
+#[allow(deprecated)] // trim_right_match -> trim_end_match
 impl Hash for BigDecimal {
     fn hash<H: Hasher>(&self, state: &mut H) {
         let mut dec_str = self.int_val.to_str_radix(10).to_string();
@@ -959,7 +996,8 @@ impl<'a> AddAssign<&'a BigInt> for BigDecimal {
         match self.scale.cmp(&0) {
             Ordering::Equal => self.int_val += rhs,
             Ordering::Greater => self.int_val += rhs * ten_to_the(self.scale as u64),
-            Ordering::Less =>  { // *self += BigDecimal::new(rhs, 0)
+            Ordering::Less => {
+                // *self += BigDecimal::new(rhs, 0)
                 self.int_val *= ten_to_the((-self.scale) as u64);
                 self.int_val += rhs;
                 self.scale = 0;
@@ -1211,8 +1249,6 @@ impl<'a, 'b> Mul<&'a BigInt> for &'b BigDecimal {
     }
 }
 
-
-
 forward_val_assignop!(impl MulAssign for BigDecimal, mul_assign);
 
 impl<'a> MulAssign<&'a BigDecimal> for BigDecimal {
@@ -1240,7 +1276,7 @@ fn impl_division(mut num: BigInt, den: &BigInt, mut scale: i64, max_precision: u
     }
 
     // shift digits until numerator is larger than denominator (set scale appropriately)
-    while &num < den {
+    while num < *den {
         scale += 1;
         num *= 10;
     }
@@ -1294,7 +1330,7 @@ impl Div<BigDecimal> for BigDecimal {
 
         let scale = self.scale - other.scale;
 
-        if &self.int_val == &other.int_val {
+        if self.int_val == other.int_val {
             return BigDecimal {
                 int_val: 1.into(),
                 scale: scale,
@@ -1320,7 +1356,7 @@ impl<'a> Div<&'a BigDecimal> for BigDecimal {
 
         let scale = self.scale - other.scale;
 
-        if &self.int_val == &other.int_val {
+        if self.int_val == other.int_val {
             return BigDecimal {
                 int_val: 1.into(),
                 scale: scale,
@@ -1500,14 +1536,14 @@ impl Signed for BigDecimal {
 
 impl Sum for BigDecimal {
     #[inline]
-    fn sum<I: Iterator<Item=BigDecimal>>(iter: I) -> BigDecimal {
+    fn sum<I: Iterator<Item = BigDecimal>>(iter: I) -> BigDecimal {
         iter.fold(Zero::zero(), |a, b| a + b)
     }
 }
 
 impl<'a> Sum<&'a BigDecimal> for BigDecimal {
     #[inline]
-    fn sum<I: Iterator<Item=&'a BigDecimal>>(iter: I) -> BigDecimal {
+    fn sum<I: Iterator<Item = &'a BigDecimal>>(iter: I) -> BigDecimal {
         iter.fold(Zero::zero(), |a, b| a + b)
     }
 }
@@ -1608,7 +1644,7 @@ impl Num for BigDecimal {
                     _ => exp,
                 };
 
-                (base, try!(i64::from_str(exp)))
+                (base, i64::from_str(exp)?)
             }
         };
 
@@ -1638,7 +1674,7 @@ impl Num for BigDecimal {
         };
 
         let scale = decimal_offset - exponent_value;
-        let big_int = try!(BigInt::from_str_radix(&digits, radix));
+        let big_int = BigInt::from_str_radix(&digits, radix)?;
 
         Ok(BigDecimal::new(big_int, scale))
     }
@@ -1660,9 +1696,7 @@ impl ToPrimitive for BigDecimal {
     }
 
     fn to_f64(&self) -> Option<f64> {
-        self.int_val
-            .to_f64()
-            .map(|x| x * 10f64.powi(-self.scale as i32))
+        self.int_val.to_f64().map(|x| x * 10f64.powi(-self.scale as i32))
     }
 }
 
@@ -1710,6 +1744,7 @@ macro_rules! impl_from_type {
     ($FromType:ty, $AsType:ty) => {
         impl From<$FromType> for BigDecimal {
             #[inline]
+            #[allow(clippy::cast_lossless)]
             fn from(n: $FromType) -> Self {
                 BigDecimal::from(n as $AsType)
             }
@@ -1725,25 +1760,21 @@ impl_from_type!(i8, i64);
 impl_from_type!(i16, i64);
 impl_from_type!(i32, i64);
 
-impl From<f32> for BigDecimal {
+impl TryFrom<f32> for BigDecimal {
+    type Error = ParseBigDecimalError;
+
     #[inline]
-    fn from(n: f32) -> Self {
-        BigDecimal::from_str(&format!(
-            "{:.PRECISION$e}",
-            n,
-            PRECISION = ::std::f32::DIGITS as usize
-        )).unwrap()
+    fn try_from(n: f32) -> Result<Self, Self::Error> {
+        BigDecimal::from_str(&format!("{:.PRECISION$e}", n, PRECISION = ::std::f32::DIGITS as usize))
     }
 }
 
-impl From<f64> for BigDecimal {
+impl TryFrom<f64> for BigDecimal {
+    type Error = ParseBigDecimalError;
+
     #[inline]
-    fn from(n: f64) -> Self {
-        BigDecimal::from_str(&format!(
-            "{:.PRECISION$e}",
-            n,
-            PRECISION = ::std::f64::DIGITS as usize
-        )).unwrap()
+    fn try_from(n: f64) -> Result<Self, Self::Error> {
+        BigDecimal::from_str(&format!("{:.PRECISION$e}", n, PRECISION = ::std::f64::DIGITS as usize))
     }
 }
 
@@ -1760,12 +1791,12 @@ impl FromPrimitive for BigDecimal {
 
     #[inline]
     fn from_f32(n: f32) -> Option<Self> {
-        Some(BigDecimal::from(n))
+        BigDecimal::try_from(n).ok()
     }
 
     #[inline]
     fn from_f64(n: f64) -> Option<Self> {
-        Some(BigDecimal::from(n))
+        BigDecimal::try_from(n).ok()
     }
 }
 
@@ -1780,7 +1811,11 @@ impl ToBigInt for BigDecimal {
 mod bigdecimal_serde {
     use super::BigDecimal;
     use serde::{de, ser};
+    use std::convert::TryFrom;
     use std::fmt;
+    use std::str::FromStr;
+    #[allow(unused_imports)]
+    use num_traits::FromPrimitive;
 
     impl ser::Serialize for BigDecimal {
         fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
@@ -1804,7 +1839,6 @@ mod bigdecimal_serde {
         where
             E: de::Error,
         {
-            use std::str::FromStr;
             BigDecimal::from_str(value).map_err(|err| E::custom(format!("{}", err)))
         }
 
@@ -1826,10 +1860,11 @@ mod bigdecimal_serde {
         where
             E: de::Error,
         {
-            Ok(BigDecimal::from(value))
+            BigDecimal::try_from(value).map_err(|err| E::custom(format!("{}", err)))
         }
     }
 
+    #[cfg(not(feature = "string-only"))]
     impl<'de> de::Deserialize<'de> for BigDecimal {
         fn deserialize<D>(d: D) -> Result<Self, D::Error>
         where
@@ -1839,6 +1874,16 @@ mod bigdecimal_serde {
         }
     }
 
+    #[cfg(feature = "string-only")]
+    impl<'de> de::Deserialize<'de> for BigDecimal {
+        fn deserialize<D>(d: D) -> Result<Self, D::Error>
+        where
+            D: de::Deserializer<'de>,
+        {
+            d.deserialize_str(BigDecimalVisitor)
+        }
+    }
+
     #[cfg(test)]
     extern crate serde_json;
 
@@ -1897,22 +1942,19 @@ mod bigdecimal_serde {
     }
 
     #[test]
+    #[cfg(not(feature = "string-only"))]
     fn test_serde_deserialize_int() {
-        use traits::FromPrimitive;
-
         let vals = vec![0, 1, 81516161, -370, -8, -99999999999];
         for n in vals {
             let expected = BigDecimal::from_i64(n).unwrap();
-            let value: BigDecimal =
-                serde_json::from_str(&serde_json::to_string(&n).unwrap()).unwrap();
+            let value: BigDecimal = serde_json::from_str(&serde_json::to_string(&n).unwrap()).unwrap();
             assert_eq!(expected, value);
         }
     }
 
     #[test]
+    #[cfg(not(feature = "string-only"))]
     fn test_serde_deserialize_f64() {
-        use traits::FromPrimitive;
-
         let vals = vec![
             1.0,
             0.5,
@@ -1928,18 +1970,18 @@ mod bigdecimal_serde {
         ];
         for n in vals {
             let expected = BigDecimal::from_f64(n).unwrap();
-            let value: BigDecimal =
-                serde_json::from_str(&serde_json::to_string(&n).unwrap()).unwrap();
+            let value: BigDecimal = serde_json::from_str(&serde_json::to_string(&n).unwrap()).unwrap();
             assert_eq!(expected, value);
         }
     }
 }
 
-#[cfg_attr(rustfmt, rustfmt_skip)]
+#[rustfmt::skip]
 #[cfg(test)]
 mod bigdecimal_tests {
     use BigDecimal;
-    use traits::{ToPrimitive, FromPrimitive};
+    use num_traits::{ToPrimitive, FromPrimitive, Signed, Zero, One};
+    use std::convert::TryFrom;
     use std::str::FromStr;
     use num_bigint;
 
@@ -1957,6 +1999,20 @@ mod bigdecimal_tests {
         assert_eq!(expected_sum, sum);
     }
 
+    #[test]
+    fn test_sum1() {
+        let vals = vec![
+            BigDecimal::from_f32(0.1).unwrap(),
+            BigDecimal::from_f32(0.2).unwrap(),
+            // BigDecimal::from_f32(0.001).unwrap(),
+        ];
+
+        let expected_sum = BigDecimal::from_f32(0.3).unwrap();
+        let sum = vals.iter().sum::<BigDecimal>();
+
+        assert_eq!(expected_sum, sum);
+    }
+
     #[test]
     fn test_to_i64() {
         let vals = vec![
@@ -2060,6 +2116,12 @@ mod bigdecimal_tests {
         }
     }
 
+    #[test]
+    fn test_nan_float() {
+        assert!(BigDecimal::try_from(std::f32::NAN).is_err());
+        assert!(BigDecimal::try_from(std::f64::NAN).is_err());
+    }
+
     #[test]
     fn test_add() {
         let vals = vec![
@@ -2199,7 +2261,6 @@ mod bigdecimal_tests {
     #[test]
     #[should_panic(expected = "Division by zero")]
     fn test_division_by_zero_panics_v2() {
-        use traits::Zero;
         let x = BigDecimal::from_str("3.14").unwrap();
         let _r = x / BigDecimal::zero();
     }
@@ -2509,6 +2570,34 @@ mod bigdecimal_tests {
         }
     }
 
+    #[test]
+    fn test_round() {
+        let test_cases = vec![
+            ("1.45", 1, "1.5"),
+            ("1.444445", 1, "1.4"),
+            ("1.44", 1, "1.4"),
+            ("0.444", 2, "0.44"),
+            ("0.0045", 2, "0.00"),
+            ("-1.555", 2, "-1.56"),
+            ("-1.555", 99, "-1.555"),
+            ("5.5", 0, "6"),
+            ("-1", -1, "0"),
+            ("5", -1, "10"),
+            ("44", -1, "40"),
+            ("44", -99, "0"),
+            ("1.4499999999", 1, "1.4"),
+            ("-1.4499999999", 1, "-1.4"),
+            ("1.449999999", 1, "1.4"),
+            ("-9999.444455556666", 10, "-9999.4444555567"),
+            ("-12345678987654321.123456789", 8, "-12345678987654321.12345679"),
+        ];
+        for &(x, digits, y) in test_cases.iter() {
+            let a = BigDecimal::from_str(x).unwrap();
+            let b = BigDecimal::from_str(y).unwrap();
+            assert_eq!(a.round(digits), b);
+        }
+    }
+
     #[test]
     fn test_is_integer() {
         let true_vals = vec![
@@ -2781,7 +2870,6 @@ mod bigdecimal_tests {
 
     #[test]
     fn test_signed() {
-        use traits::{One, Signed, Zero};
         assert!(!BigDecimal::zero().is_positive());
         assert!(!BigDecimal::one().is_negative());
 
@@ -2790,6 +2878,35 @@ mod bigdecimal_tests {
         assert!((-BigDecimal::one()).abs().is_positive());
     }
 
+    #[test]
+    fn test_normalize() {
+        use num_bigint::BigInt;
+
+        let vals = vec![
+            (BigDecimal::new(BigInt::from(10), 2),
+            BigDecimal::new(BigInt::from(1), 1),
+            "0.1"),
+            (BigDecimal::new(BigInt::from(132400), -4),
+            BigDecimal::new(BigInt::from(1324), -6),
+            "1324000000"),
+            (BigDecimal::new(BigInt::from(1_900_000), 3),
+            BigDecimal::new(BigInt::from(19), -2),
+            "1900"),
+            (BigDecimal::new(BigInt::from(0), -3),
+            BigDecimal::zero(),
+            "0"),
+            (BigDecimal::new(BigInt::from(0), 5),
+            BigDecimal::zero(),
+            "0"),
+        ];
+
+        for (not_normalized, normalized, string) in vals {
+            assert_eq!(not_normalized.normalized(), normalized);
+            assert_eq!(not_normalized.normalized().to_string(), string);
+            assert_eq!(normalized.to_string(), string);
+        }
+    }
+
     #[test]
     #[should_panic(expected = "InvalidDigit")]
     fn test_bad_string_nan() {
